    ;; YaPPP 0.7 - Yet Another PPP.  Based on Thomas Neuhaus' PPP.
    ;; Copyright (C) 2013 Emanuel Wittersheim

    ;; This program is free software: you can redistribute it and/or modify
    ;; it under the terms of the GNU General Public License as published by
    ;; the Free Software Foundation, either version 3 of the License, or
    ;; (at your option) any later version.

    ;; This program is distributed in the hope that it will be useful,
    ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
    ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    ;; GNU General Public License for more details.

    ;; You should have received a copy of the GNU General Public License
    ;; along with this program.


;;--- Verfügbare Unit Generators:
;;    RandomMaker
;;    JitterPositionMaker
;;    RandomWalkMaker 
;;    TendencyMaker
;;    LineMaker
;;    ExptLineMaker 
;;    ShuffleListMaker
;;    PatternMaker
;;    AccumulatorMaker
;;    FindNextMaker
;;    ChangeMaker
;;    ObjectMaker

;; Gibt einen Zufallswert zwischen den gegebenen Grenzen ug und og aus:
(define (RandomMaker init-ug init-og)
  (let ((ug (box-maker init-ug))
	(og (box-maker init-og)))
    (lambda msg
      (case (car msg)
	((type) "Random within given ranges")
	((next) 
	 (let ((next-value (+ (random (- (send og 'show) (send ug 'show)))
			      (send ug 'show))))
	   next-value))
	(else
	 (delegate base-object msg))))))

;; Gibt einen ZufallsWert, der im Intervall Max-step um pos bewegt, aus:
(define (JitterPositionMaker init-pos init-max-step)
  (let* ((pos (box-maker init-pos))
	 (max-step (box-maker init-max-step))
	 (rand (RandomMaker (* -1.0 (send max-step 'show))
			     (send max-step 'show))))
    (lambda msg
      (case (car msg)
	((type) "Jitter Position")
	((next) (let ((next-rand (+ (send rand 'next) 
				    (send pos 'show))))
		  next-rand))
	(else (delegate base-object msg))))))

;; Gibt einen Zufallswert aus, der sich mit jedem next um maximal max-step weiterbewegt:
(define (RandomWalkMaker init-pos init-max-step)
  (let ((pos (box-maker init-pos))
	(max-step (box-maker init-max-step))
	(rand (JitterPositionMaker init-pos init-max-step)))
    (lambda msg
      (case (car msg)
	((type) "Random Walk")
	((next) (let ((next-rand (send rand 'next)))
		  (set! rand (JitterPositionMaker next-rand (send max-step 'show)))
		  (send pos 'update! next-rand)
		  (send pos 'show)))
	((reset!) (send pos 'reset!)
	 (send pos 'show))
	(else (delegate base-object msg))))))

;; Gibt einen Wert aus, der sich auf einer definierten lienie bewegt:
(define (LineMaker x0 y0 x1 y1)
  (let* ((x (box-maker x0))
	 (y (box-maker y0))
	 (m (box-maker (/ (- y1 y0)(- x1 x0))))
	 (b (box-maker (* -1.0 (- (* (send m 'show) x0) y0)))))
    (lambda msg
      (case (car msg)
	((type) "Straight Line Maker")
	((next) (let* ((next_y (+ (* (send m 'show) (cadr msg)) (send b 'show))))
		  next_y))
	((reset!) (send x 'reset!)
	 (send y 'reset!)
	 (send m 'reset!)
	 (send b 'reset!))
	(else (delegate base-object msg))))))

;;(define test (LineMaker 0.0 1.0 10.0 20.0))
;;(send test 'next 10.0)

;; Gibt einen Wert aus, der sich auf einer Poten-Funktion befindet :
(define (ExptLineMaker init-a init-r)
  (let* ((a (box-maker init-a))
	 (r (box-maker init-r))
	 (y (box-maker 0.0))
	 (x (box-maker 0.0)))
    (lambda msg
      (case (car msg)
	((type) "Power Function")
	((next) (let ((next_y (expt (* (send a 'show) (cadr msg)) 
				    (send r 'show))))
		  next_y))
	((reset!) (send a 'reset!)
	 (send r 'reset!)
	 (send y 'reset!)
	 (send x 'reset!))
	(else
	 (delegate base-object msg))))))

;; Erstellt eine Tendenz-Maske deren Grenzen entweder Potenzfunktionen oder lineare sind:
(define (TendencyMaker)
  (lambda (line-proc . args)
    (let ((ug (apply line-proc (car args)))
	  (og (apply line-proc (cadr args))))
      (lambda msg
	(case (car msg)
	  ((type) "Tendency Maker")
	  ((next) (let* ((this-ug (send ug 'next (cadr msg)))
			 (this-og (send og 'next (cadr msg)))
			 (this-rand (RandomMaker this-ug this-og)))
		    (send this-rand 'next)))
	  ((reset!)
	   (send ug 'reset!)
	   (send og 'reset!))
	  (else (delegate base-object msg)))))))

;; Mist eine Liste und gibt bei jedem next ein Element der Liste zurück
(define (ShuffleListMaker list)
  (let ((len (- (length list) 1))
	(last-rnd (box-maker 'undefined)))
    (lambda msg
      (case (car msg)
	((type) "List Shuffler")
	((last) (send last-rnd 'show))
	((next) (let ((this-rnd (list-ref list (random len))))
		  (send last-rnd 'update! this-rnd)
		  (send last-rnd 'show)))
	(else (delegate base-object msg))))))

;; Wiederholt die Liste bei jedem next
(define (PatternMaker list)
  (let ((len (length list))
	(counter (box-maker 0))
	(last-elem (box-maker 'undefined)))
    (lambda msg
      (case (car msg)
	((type) "Pattern Maker")
	((next) (let* ((next-count (+ (send counter 'show) 1))
		       (next-elem (list-ref list (modulo next-count len))))
		  (send counter 'update! next-count)
		  (send last-elem 'update! next-elem)
		  (send last-elem 'show)))
	((last) (send last-elem 'show))
	((reset!) (send counter 'reset!)
	 (send last-elem 'reset!))
	(else (delegate base-object msg))))))

;; Addiert bei jedem next den neuen auf den alten Wert und gibt diesen zurück
(define (AccumulatorMaker init-val change-proc)
  (let ((value (box-maker init-val)))
    (lambda msg
      (case (car msg)
	((type) "Accumulator Maker")
	((update!) (let ((next-value (change-proc (send value 'show) (cadr msg))))
		     (send value 'update! next-value)
		     (send value 'show)))
	((show) (send value 'show))
	((reset!) (send 'reset! value))
	(else (delegate base-object msg))))))

;; Findet das nächste Element in einer liste zu einem passenden element (VORSICHTIG!!!)
(define (FindNextMaker list)
  (lambda (elem)
    (let ((ls (box-maker list)))
      (lambda msg
	(case (car msg)
	  ((type) "Find Next Maker")
	  ((next) (let ((next-elem (find_next elem (send ls 'show))))
		    next-elem))
	  (else (delegate base-object msg)))))))

(define (find_next elem list)
  (cond ((null? list) (error "Something went Wrong!"))
	((= elem (car list)) (cadr list))
	(else (find_next elem (cdr list)))))


(define ChangeMaker
  (lambda (times . procs)
    (let* ((count (box-maker 0))
	   (akt_proc (box-maker (list-ref procs (send count 'show))))
	   (check_time (box-maker (list-ref times (send count 'show)))))
      (lambda msg
	(case (car msg)
	  ((type) "Change by time")
	  ((next)
	   (cond ((< (cadr msg) (send check_time 'show))
		  (let ((this_proc (send akt_proc 'show)))
		    (send this_proc 'next (cadr msg))))
		 (else (let ((this_count (send count 'show)))
			 (send count 'update! (+ this_count 1))
			 (send akt_proc 'update! (list-ref procs (send count 'show)))
			 (send check_time 'update! (list-ref times (send count 'show)))
			 (send (send akt_proc 'show) 'next (cadr msg))))))
	  (else (delegate base-object msg)))))))

(define ChangeTrackMaker
  (lambda procs
    (let ((this_procs (box-maker procs)))
      (lambda msg
	(case (car msg)
	  ((type) "Change for Tracks")
	  ((next) (let ((akt_proc (list-ref (send this_procs 'show) (cadr msg))))
		    (send akt_proc 'next (send TIME 'show))))
	  (else
	   (delegate base-object msg)))))))
      

(define ObjectMaker 
  (lambda (proc . args)
    (let ((this_proc (box-maker proc))
	  (this_args (box-maker args)))
      (lambda msg
	(case (car msg)
	  ((type) "Oject Maker")
	  ((next) (cond ((null? args) 
			 (let ((apl (send this_proc 'show)))
			   (apl)))
			(else (apply (send this_proc 'show) args))))
	  (else (delegate base-object msg)))))))

(define Interpolate 
  (lambda (from to . args)
    (let* ((from_proc (apply from (car args)))
           (to_proc (apply to (cadr args)))
           (t_strt (send TIME 'show))
           (t_end (send MAX_TIME 'show))
           (checker ((TendencyMaker)
                     LineMaker `(,t_strt 0.0 ,t_end 0.5)
                     `(,t_strt 0.5 ,t_end 1.0))))
      (lambda msg
        (case (car msg)
          ((type) "Interpolation")
          ((next) (let ((check (send checker 'next (cadr msg))))
                    (if (> check  0.5)
                        (send to_proc 'next (cadr msg))
                        (send from_proc 'next (cadr msg)))))
          (else (delegate base-object msg)))))))
  


(newline)
(display "YaPPP 0.7 Copyright (C) 2013 Emanuel Wittersheim
	This program comes with ABSOLUTELY NO WARRANTY. For details see file COPYING. 
	This is free software, and you are welcome to redistribute it
	under certain conditions.")
(newline)
